"""
Fragments of label mutations
"""

LABEL_FRAGMENT = '''
id
author {
  id
  email
}
labelType
jsonResponse
createdAt
secondsToLabel
totalSecondsToLabel
honeypotMark
'''


LABEL_FRAGMENT_ID = '''
id
'''
